@@ -1,43 +1,61 @@
 #!/usr/bin/env perl
-# $Id: updmap.pl 42939 2017-01-13 14:32:10Z preining $
+# $Id: updmap.pl 77314 2026-01-09 16:20:28Z karl $
 # updmap - maintain map files for outline fonts.
 # (Maintained in TeX Live:Master/texmf-dist/scripts/texlive.)
 # 
-# Copyright 2011-2016 Norbert Preining
+# Copyright 2011-2024 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 #
 # History:
-# Original shell script (C) 2002 Thomas Esser
+# Original shell script (C) 2002 Thomas Esser, released to the public domain.
 # first perl variant by Fabrice Popineau
-# later adaptions by Reinhard Kotucha and Karl Berry
-# the original versions were licensed under the following agreement:
-# Anyone may freely use, modify, and/or distribute this file, without
+# later adaptions by Reinhard Kotucha, and Karl Berry.
+# The current implementation is a complete rewrite.
 
-my $svnid = '$Id: updmap.pl 42939 2017-01-13 14:32:10Z preining $';
+my $svnid = '$Id: updmap.pl 77314 2026-01-09 16:20:28Z karl $';
+
+use strict; use warnings;
 
 my $TEXMFROOT;
 BEGIN {
   $^W = 1;
+  # make subprograms (including kpsewhich) have the right path:
+  my $bindir;
+  my $Master = __FILE__;
+  if ($^O =~ /^MSWin/i) {
+    # on w32 $0 and __FILE__ point directly to this script; they can be relative
+    $Master =~ s!\\!/!g;
+    $Master =~ s![^/]*$!../../..!
+      unless ($Master =~ s!/texmf-dist/scripts/texlive/tlmgr\.pl$!!i);
+    $bindir = "$Master/bin/windows";
+  } else {
+    $Master =~ s,/*[^/]*$,,;
+    $bindir = $Master;
+    $Master = "$Master/../..";
+  }
+  $ENV{"PATH"} = "$bindir:$ENV{PATH}";
   $TEXMFROOT = `kpsewhich -var-value=TEXMFROOT`;
-  if ($?) {
-    die "$0: kpsewhich -var-value=TEXMFROOT failed, aborting early.\n";
+  if ($? || ! $TEXMFROOT) {
+    warn "$0: kpsewhich -var-value=TEXMFROOT failed, aborting early.\n";
+    warn "$0:   got TEXMFROOT value: $TEXMFROOT" if $TEXMFROOT;
+    die  "$0:   had PATH: $ENV{PATH}\n";
   }
   chomp($TEXMFROOT);
   unshift(@INC, "$TEXMFROOT/tlpkg");
 }
 
-my $lastchdate = '$Date: 2017-01-13 15:32:10 +0100 (Fri, 13 Jan 2017) $';
+my $lastchdate = '$Date: 2026-01-09 17:20:28 +0100 (Fri, 09 Jan 2026) $';
 $lastchdate =~ s/^\$Date:\s*//;
 $lastchdate =~ s/ \(.*$//;
-my $svnrev = '$Revision: 42939 $';
+my $svnrev = '$Revision: 77314 $';
 $svnrev =~ s/^\$Revision:\s*//;
 $svnrev =~ s/\s*\$$//;
 my $version = "r$svnrev ($lastchdate)";
 
 use Getopt::Long qw(:config no_autoabbrev ignore_case_always);
 use strict;
-use TeXLive::TLUtils qw(mkdirhier mktexupd win32 basename dirname 
+use TeXLive::TLUtils qw(mkdirhier mktexupd wndws basename dirname 
   sort_uniq member touch);
 
 (my $prg = basename($0)) =~ s/\.pl$//;
@@ -54,7 +72,7 @@ chomp(my $TEXMFSYSCONFIG = `kpsewhich -var-value=TEXMFSYSCONFIG`);
 chomp(my $TEXMFHOME = `kpsewhich -var-value=TEXMFHOME`);
 
 # make sure that on windows *everything* is in lower case for comparison
-if (win32()) {
+if (wndws()) {
   $TEXMFDIST = lc($TEXMFDIST);
   $TEXMFVAR = lc($TEXMFVAR);
   $TEXMFSYSVAR = lc($TEXMFSYSVAR);
@@ -67,20 +85,22 @@ if (win32()) {
 my $texmfconfig = $TEXMFCONFIG;
 my $texmfvar    = $TEXMFVAR;
 
-my %opts = ( quiet => 0, nohash => 0, nomkmap => 0 );
+# copy by default for portability.
+my %opts = ( quiet => 0, nohash => 0, nomkmap => 0, copy => 1 );
 my $alldata;
 my $updLSR;
 
 my @cmdline_options = (
   "sys",
+  "user",
   "listfiles",
   "cnffile=s@", 
-  "copy", 
+  "copy!", 
   "disable=s@",
   "dvipdfmoutputdir=s",
   "dvipdfmxoutputdir=s",
   "dvipsoutputdir=s",
-  # the following does not work, Getopt::Long looses the first
+  # the following does not work, Getopt::Long loses the first
   # entry in a multi setting, treat it separately in processOptions
   # furthermore, it is not supported by older perls, so do it differently
   #"enable=s{1,2}",
@@ -88,8 +108,8 @@ my @cmdline_options = (
   "force",
   "listavailablemaps",
   "listmaps|l",
-  "nohash",
-  "nomkmap",
+  "nohash!",
+  "nomkmap!",
   "dry-run|n",
   "outputdir=s",
   "pdftexoutputdir=s",
@@ -172,27 +192,9 @@ sub main {
     exit (0);
   }
 
-  # check if we are in *hidden* sys mode, in which case we switch
-  # to sys mode
-  # Nowdays we use -sys switch instead of simply overriding TEXMFVAR
-  # and TEXMFCONFIG
-  # This is used to warn users when they run updmap in usermode the first time.
-  # But it might happen that this script is called via another wrapper that
-  # sets TEXMFCONFIG and TEXMFVAR, and does not pass on the -sys option.
-  # for this case we check whether the SYS and non-SYS variants agree,
-  # and if, then switch to sys mode (with a warning)
-  if (($TEXMFSYSCONFIG eq $TEXMFCONFIG) && ($TEXMFSYSVAR eq $TEXMFVAR)) {
-    if (!$opts{'sys'}) {
-      print_warning("hidden sys mode found, switching to sys mode.\n");
-      $opts{'sys'} = 1;
-    }
-  }
-
-  if ($opts{'sys'}) {
-    # we are running as updmap-sys, make sure that the right tree is used
-    $texmfconfig = $TEXMFSYSCONFIG;
-    $texmfvar    = $TEXMFSYSVAR;
-  }
+  ($texmfconfig, $texmfvar) = 
+    TeXLive::TLUtils::setup_sys_user_mode($prg, \%opts,
+      $TEXMFCONFIG, $TEXMFSYSCONFIG, $TEXMFVAR, $TEXMFSYSVAR);
 
   if ($opts{'dvipdfmoutputdir'} && !defined($opts{'dvipdfmxoutputdir'})) {
     $opts{'dvipdfmxoutputdir'} = $opts{'dvipdfmoutputdir'};
@@ -249,7 +251,7 @@ sub main {
       if (! -f $f) {
         die "$prg: Config file \"$f\" not found.";
       }
-      push @tmp, (win32() ? lc($f) : $f);
+      push @tmp, (wndws() ? lc($f) : $f);
     }
     @{$opts{'cnffile'}} = @tmp;
     # in case that config files are given on the command line, the first
@@ -260,12 +262,12 @@ sub main {
     chomp(@all_files);
     my @used_files;
     for my $f (@all_files) {
-      push @used_files, (win32() ? lc($f) : $f);
+      push @used_files, (wndws() ? lc($f) : $f);
     }
     #
     my $TEXMFLOCALVAR;
     my @TEXMFLOCAL;
-    if (win32()) {
+    if (wndws()) {
       chomp($TEXMFLOCALVAR =`kpsewhich --expand-path=\$TEXMFLOCAL`);
       @TEXMFLOCAL = map { lc } split(/;/ , $TEXMFLOCALVAR);
     } else {
@@ -439,7 +441,7 @@ sub main {
     # but for compatibility we'll silently keep the option.
     $cmd = 'edit';
     my $editor = $ENV{'VISUAL'} || $ENV{'EDITOR'};
-    $editor ||= (&win32 ? "notepad" : "vi");
+    $editor ||= (wndws() ? "notepad" : "vi");
     if (-r $changes_config_file) {
       &copyFile($changes_config_file, $bakFile);
     } else {
@@ -498,7 +500,8 @@ sub main {
   }
 
   if (!$opts{'nohash'}) {
-    print "$prg: Updating ls-R files.\n" if !$opts{'quiet'};
+    my $not = $opts{"dry-run"} ? " not (-n)" : "";
+    print "$prg:$not updating ls-R files.\n" if !$opts{'quiet'};
     $updLSR->{exec}() unless $opts{"dry-run"};
   }
 
@@ -610,7 +613,7 @@ sub setupSymlinks {
 sub SymlinkOrCopy {
   my ($dir, $src, $dest) = @_;
   return ($src, $dest) if $opts{"dry-run"};
-  if (&win32 || $opts{'copy'}) {  # always copy
+  if (wndws() || $opts{'copy'}) {  # always copy
     &copyFile("$dir/$src", "$dir/$dest");
   } else { # symlink if supported by fs, copy otherwise
     system("cd \"$dir\" && ln -s $src $dest 2>/dev/null || "
@@ -862,9 +865,18 @@ sub cidx2dvips {
     chomp;
     # save the line for warnings
     my $l = $_;
-    # first check whether a PSname is given
-    my $psname;
     #
+    my $psname;
+    my $fbname;
+    #
+    # special case for pre-defined fallback from unicode encoded font
+    if ($_ =~ m/%!DVIPSFB\s\s*([0-9A-Za-z-_!,][0-9A-Za-z-_!,]*)/) {
+      $fbname = $1;
+      # minimal adjustment
+      $fbname =~ s/^!//;
+      $fbname =~ s/,Bold//;
+    }
+    # first check whether a PSname is given
     # the matching on \w* is greedy, so will take all the word chars available
     # that means we do not need to test for end of word
     if ($_ =~ m/%!PS\s\s*([0-9A-Za-z-_][0-9A-Za-z-_]*)/) {
@@ -883,12 +895,17 @@ sub cidx2dvips {
       $italicmax = .3;
       s/,Italic//;
     }
+    # replace supported "/AJ16" and co. for ptex-fontmaps CID emulation
+    # note that the emulation method in GS is incomplete
+    # due to "Reversal CMap method" (cf. "ToUnicode method")
+    s!/A[JGCK]1[0-6]!!;
     # break out if unsupported constructs are found: @ / ,
     next if (m![\@/,]!);
     # make everything single spaced
     s/\s\s*/ /g;
     # unicode encoded fonts are not supported
-    next if (m!^\w\w* unicode !);
+    # but if a fallback font is pre-defined, we can use it
+    next if (!defined($fbname) && (m!^[0-9A-Za-z-_][0-9A-Za-z-_]* unicode !));
     # now we have the following format
     #  <word> <word> <word> some options like -e or -s
     if ($_ !~ m/([^ ][^ ]*) ([^ ][^ ]*) ([^ ][^ ]*)( (.*))?$/) {
@@ -924,13 +941,17 @@ sub cidx2dvips {
       $opts .= " \"$italicmax SlantFont\"";
     }
     # print out the result
-    if (defined($psname)) {
-      push @d, "$tfmname $psname-$cid$opts\n";
+    if (defined($fbname)) {
+      push @d, "$tfmname $fbname\n";
     } else {
-      if (defined($fname_psname{$fname})) {
-        push @d, "$tfmname $fname_psname{$fname}-$cid$opts\n";
+      if (defined($psname)) {
+        push @d, "$tfmname $psname-$cid$opts\n";
       } else {
-        push @d, "$tfmname $fname-$cid$opts\n";
+        if (defined($fname_psname{$fname})) {
+          push @d, "$tfmname $fname_psname{$fname}-$cid$opts\n";
+        } else {
+          push @d, "$tfmname $fname-$cid$opts\n";
+        }
       }
     }
   }
@@ -1025,7 +1046,8 @@ sub mkMaps {
   # directory unless we are going to put something there.
   setupOutputDir("pxdvi") if $pxdviUse eq "true";
 
-  print_and_log ("\n$prg is creating new map files"
+  my $not = $opts{"dry-run"} ? " not (-n)" : "";
+  print_and_log ("\n$prg is$not creating new map files"
          . "\nusing the following configuration:"
          . "\n  LW35 font names                  : "
          .      "$mode ($mode_origin)"
@@ -1247,7 +1269,7 @@ sub mkMaps {
 
   our $link = &setupSymlinks($dvipsPreferOutline, $dvipsoutputdir, $pdftexDownloadBase14, $pdftexoutputdir);
 
-  print_and_log ("\nFiles generated:\n");
+  print_and_log ("\nFiles$not generated:\n");
   sub dir {
     my ($d, $f, $target)=@_;
     our $link;
@@ -1337,19 +1359,27 @@ sub mkMaps {
     }
   }
 
-  # all kind of warning messages
+  # all kinds of warning messages
   if ($first_time_creation_in_usermode) {
     print_and_log("
-WARNING: you are switching to updmap's per-user mappings.
+*************************************************************
+*                                                           *
+* WARNING: you are switching to updmap's per-user mappings. *
+*         Please read the following warnings!               *
+*                                                           *
+*************************************************************
 
-You have run updmap (as opposed to updmap-sys) for the first time; this
-has created configuration files which are local to your personal account.
+You have run updmap-user (as opposed to updmap-sys) for the first time;
+this has created configuration files which are local to your personal account.
 
-Any changes in system map files will *not* be automatically reflected in
-your files; furthermore, running updmap-sys will no longer have any
-effect for you.  As a consequence, you have to rerun updmap yourself
-after any change in the system directories; for example, if a new font
-package is added.
+From now on, any changes in system map files will *not* be automatically
+reflected in your files; furthermore, running updmap-sys (as is done
+automatically) will no longer have any effect for you.
+
+As a consequence, you yourself have to rerun updmap-user yourself after
+any change in the *system* directories! For example, if a new font
+package is added or existing mappings change, which happens frequently.
+See https://tug.org/texlive/scripts-sys-user.html for details.
 
 If you want to undo this, remove the files mentioned above.
 
@@ -1365,10 +1395,10 @@ The following files have been generated as listed above,
 but will not be found because overriding files exist, listed below.
 ");
     #
-    if ($prg eq "updmap-sys") {
+    if ($opts{'sys'}) {
       print_and_log ("
-Perhaps you have run updmap in the past, but are running updmap-sys
-now.  Once you run updmap the first time, you have to keep using it,
+Perhaps you have run updmap-user in the past, but are running updmap-sys
+now.  Once you run updmap-user the first time, you have to keep using it,
 or else remove the personal configuration files it creates (the ones
 listed below).
 ");
@@ -1382,8 +1412,7 @@ listed below).
   }
 
   close LOG unless $opts{'dry-run'};
-  print "\nTranscript written on \"$logfile\".\n" if !$opts{'quiet'};
-
+  print "\nTranscript$not written on: $logfile\n" if !$opts{'quiet'};
 }
 
 
@@ -1491,6 +1520,11 @@ sub enable_disable_maps {
     if ($w =~ m/=/) {
       # this is --enable MapType=MapName
       my ($type, $map) = split ('=', $w);
+      # allow for all lowercase map types (map/mixedmap/kanjimap)
+      $type =~ s/map$/Map/;
+      $type = ucfirst($type);
+      # don't allow map names containing /
+      die "$prg: map names cannot contain /: $map\n" if ($map =~ m{/});
       enable_map($tc, $type, $map);
     } else {
       # this is --disable MapName
@@ -2062,7 +2096,12 @@ sub read_map_files {
   my @fullpath = `kpsewhich --format=map @maps`;
   chomp @fullpath;
   foreach my $map (@maps) {
-    my ($ff) = grep /\/$map(\.map)?$/, @fullpath;
+    # in case they give an absolute path (not needed/desired, but ...);
+    # Windows not supported.
+    my $dirsep = ($map =~ m!^/!) ? "" : "/";
+    # quotemeta the map string to avoid perl regexp warning, e.g.,
+    # if map name contains "\Users", the "\U" should be literal.
+    my ($ff) = grep /$dirsep\Q$map\E(\.map)?$/, @fullpath;
     if ($ff) {
       $alldata->{'maps'}{$map}{'fullpath'} = $ff;
     } else {
@@ -2197,7 +2236,7 @@ sub merge_data {
 #   and reset it to the real home dir of root.
 
 sub reset_root_home {
-  if (!win32() && ($> == 0)) {  # $> is effective uid
+  if (!wndws() && ($> == 0)) {  # $> is effective uid
     my $envhome = $ENV{'HOME'};
     # if $HOME isn't an existing directory, we don't care.
     if (defined($envhome) && (-d $envhome)) {
@@ -2240,40 +2279,48 @@ sub version {
 
 sub help {
   my $usage = <<"EOF";
-Usage: $prg     [OPTION] ... [COMMAND]
-   or: $prg-sys [OPTION] ... [COMMAND]
+Usage: $prg [-user|-sys] [OPTION] ... [COMMAND]
+   or: $prg-user [OPTION] ... [COMMAND]
+   or: $prg-sys  [OPTION] ... [COMMAND]
 
-Update the default font map files used by pdftex (pdftex.map), dvips
-(psfonts.map), and dvipdfm(x), and optionally pxdvi, as determined by
-all configuration files updmap.cfg (the ones returned by running
-"kpsewhich --all updmap.cfg", but see below).
+Update the default font map files used by pdftex and dvipdfm(x)
+(pdftex.map), dvips (psfonts.map), and optionally pxdvi, as determined
+by all configuration files updmap.cfg (usually the ones returned by
+running "kpsewhich --all updmap.cfg", but see below).
 
 Among other things, these map files are used to determine which fonts
 should be used as bitmaps and which as outlines, and to determine which
 font files are included, typically subsetted, in the PDF or PostScript output.
 
-updmap-sys is intended to affect the system-wide configuration, while
-updmap affects personal configuration files only, overriding the system
-files.  As a consequence, once updmap has been run, even a single time,
-running updmap-sys no longer has any effect.  (updmap-sys issues a
-warning in this situation.)
+These maps are for fonts installed within the TeX hierarchy, and are not
+related to any system font lookups. They are primarily used for Type 1
+fonts, though a few OpenType and TrueType fonts are involved also.
 
-By default, the TeX filename database (ls-R) is also updated.
+updmap-sys (or updmap -sys) is intended to affect the system-wide 
+configuration, while updmap-user (or updmap -user) affects personal
+configuration files only, overriding the system files.  
+
+As a consequence, once updmap-user has been run, even a single time,
+running updmap-sys no longer has any effect.  updmap-sys issues a
+warning about this, since it is rarely desirable.
+See https://tug.org/texlive/scripts-sys-user.html for details.
+
+By default, the TeX filename database (ls-R) is also updated; use
+--nohash to skip that step.
 
 The updmap system is regrettably complicated, for both inherent and
 historical reasons.  A general overview:
+
 - updmap.cfg files are mainly about listing other files, namely the
   font-specific .maps, in which each line gives information about a
   different TeX (.tfm) font.
 - updmap reads the updmap.cfg files and then concatenates the
-  contents of those .map files into the main output files: psfonts.map
-  for dvips and pdftex.map for pdftex and dvipdfmx.
+  contents of those .map files into the main output files, generically
+  named: psfonts.map for dvips, and pdftex.map for pdftex and dvipdfmx.
 - The updmap.cfg files themselves are created and updated at package
   installation time, by the system installer or the package manager or
   by hand, and not (by default) by updmap.
 
-Good luck.
-
 Options:
   --cnffile FILE            read FILE for the updmap configuration 
                              (can be given multiple times, in which case
@@ -2283,30 +2330,32 @@ Options:
   --pdftexoutputdir DIR     specify output directory (pdftex syntax)
   --pxdvioutputdir DIR      specify output directory (pxdvi syntax)
   --outputdir DIR           specify output directory (for all files)
-  --copy                    cp generic files rather than using symlinks
+  --[no-]copy               create generic files as copies (default);
+                               with -no-copy, create symlinks
   --force                   recreate files even if config hasn't changed
   --nomkmap                 do not recreate map files
-  --nohash                  do not run texhash
+  --nohash                  do not run mktexlsr (a.k.a. texhash)
   --sys                     affect system-wide files (equivalent to updmap-sys)
+  --user                    affect personal files (equivalent to updmap-user)
   -n, --dry-run             only show the configuration, no output
   --quiet, --silent         reduce verbosity
 
 Commands:
-  --help                    show this message and exit
-  --version                 show version information and exit
-  --showoption OPTION       show the current setting of OPTION
-  --showoptions OPTION      show possible settings for OPTION
-  --setoption OPTION VALUE  set OPTION to value; option names below
-  --setoption OPTION=VALUE  as above, just different syntax
-  --enable MAPTYPE MAPFILE  add "MAPTYPE MAPFILE" to updmap.cfg,
-                             where MAPTYPE is Map, MixedMap, or KanjiMap
-  --enable Map=MAPFILE      add \"Map MAPFILE\" to updmap.cfg
-  --enable MixedMap=MAPFILE add \"MixedMap MAPFILE\" to updmap.cfg
-  --enable KanjiMap=MAPFILE add \"KanjiMap MAPFILE\" to updmap.cfg
-  --disable MAPFILE         disable MAPFILE, of whatever type
-  --listmaps                list all maps (details below)
-  --listavailablemaps       list available maps (details below)
-  --syncwithtrees           disable unavailable map files in updmap.cfg
+  --help                     show this message and exit
+  --version                  show version information and exit
+  --showoption OPTION        show the current setting of OPTION
+  --showoptions OPTION       show possible settings for OPTION
+  --setoption OPTION VALUE   set OPTION to value; option names below
+  --setoption OPTION=VALUE   as above, just different syntax
+  --enable MAPTYPE MAPFILE   add "MAPTYPE MAPFILE" to updmap.cfg,
+                              where MAPTYPE is Map, MixedMap, or KanjiMap
+  --enable Map=MAPFILE       add \"Map MAPFILE\" to updmap.cfg
+  --enable MixedMap=MAPFILE  add \"MixedMap MAPFILE\" to updmap.cfg
+  --enable KanjiMap=MAPFILE  add \"KanjiMap MAPFILE\" to updmap.cfg
+  --disable MAPFILE          disable MAPFILE, of whatever type
+  --listmaps                 list all maps (details below)
+  --listavailablemaps        list available maps (details below)
+  --syncwithtrees            disable unavailable map files in updmap.cfg
 
 The main output:
 
@@ -2322,7 +2371,7 @@ The main output:
   only and are not like other map files.  dvipdfmx reads pdftex.map for
   the map entries for non-Kanji fonts.
   
-  If no option is given, so the invocation is just "updmap" or
+  If no option is given, so the invocation is just "updmap-user" or
   "updmap-sys", these output files are always recreated.
 
   Otherwise, if an option such as --enable or --disable is given, the
@@ -2336,7 +2385,7 @@ Explanation of the map types:
   
   The only difference between Map and MixedMap is that MixedMap entries
   are not added to psfonts_pk.map.  The purpose is to help users with
-  devices that render Type 1 outline fonts worse than mode-tuned Type 1
+  devices that render Type 1 outline fonts worse than mode-tuned Type 3
   bitmap fonts.  So, MixedMap is used for fonts that are available as
   both Type 1 and Metafont.
 
@@ -2401,7 +2450,7 @@ Explanation of trees and files normally used:
   TEXMFLOCAL     \$TEXLIVE/texmf-local/web2c/updmap.cfg
   TEXMFDIST      \$TEXLIVE/YYYY/texmf-dist/web2c/updmap.cfg
 
-  For updmap:
+  For updmap-user:
   TEXMFCONFIG    \$HOME/.texliveYYYY/texmf-config/web2c/updmap.cfg
   TEXMFVAR       \$HOME/.texliveYYYY/texmf-var/web2c/updmap.cfg
   TEXMFHOME      \$HOME/texmf/web2c/updmap.cfg
@@ -2439,7 +2488,7 @@ Where and which updmap.cfg changes are saved:
   config file, a higher-level one can be used.  That way, the
   distribution's settings can be overridden system-wide using
   TEXMFLOCAL, and system settings can be overridden again in a
-  particular user's TEXMFHOME.
+  particular user's TEXMFHOME or TEXMFCONFIG.
 
 Resolving multiple definitions of a font:
 
@@ -2483,11 +2532,11 @@ Listing of maps:
   (again separated by tab) containing '(not available)' for those
   map files that cannot be found.
  
-updmap vs. updmap-sys:
+updmap-user vs. updmap-sys:
 
   When updmap-sys is run, TEXMFSYSCONFIG and TEXMFSYSVAR are used
   instead of TEXMFCONFIG and TEXMFVAR, respectively.  This is the
-  primary difference between updmap-sys and updmap.
+  primary difference between updmap-sys and updmap-user.
 
   Other locations may be used if you give them on the command line, or
   these trees don't exist, or you are not using the original TeX Live.
@@ -2498,11 +2547,12 @@ will be read and written, give the -n option (or read the source).
 The log file is written to TEXMFVAR/web2c/updmap.log.
 
 For step-by-step instructions on making new fonts known to TeX, read
-http://tug.org/fonts/fontinstall.html.  For even more terse
+https://tug.org/fonts/fontinstall.html.  For even more terse
 instructions, read the beginning of the main updmap.cfg file.
 
+Executable location: $0
 Report bugs to: tex-live\@tug.org
-TeX Live home page: <http://tug.org/texlive/>
+TeX Live home page: <https://tug.org/texlive/>
 EOF
 ;
   print &version();
