@@ -124,7 +124,7 @@ describe Language do
 
     it "lists all languages" do
       # All the TeX files.  Note [no] and [mn-cyrl-x-lmc] don’t have corresponding plain text files.
-      expect(Language.languages.count).to eq 85 # Was 79; 3 more “TeX Live dummies” [ar] [fa] [grc-x-ibycus] TODO Maybe remove
+      expect(Language.languages.count).to eq 87 # Was 79; 3 more “TeX Live dummies” [ar] [fa] [grc-x-ibycus] TODO Maybe remove # Now 87 on 2025-02-14
     end
   end
 
@@ -133,8 +133,8 @@ describe Language do
       expect(Language.all).to be_an Array
     end
 
-    it "returns 81 languages" do # That’s all of them except for [sr-cyrl]
-      expect(Language.all.count).to eq 84
+    it "returns 86 languages" do # That’s all of them except for [sr-cyrl]
+      expect(Language.all.count).to eq 86
     end
   end
 
@@ -686,8 +686,8 @@ describe Language do
       expect(Language.new('la').isgreek?).to be_falsey
     end
 
-    it "exceptionally returns false for Ibycus" do
-      expect(Language.new('grc-x-ibycus').isgreek?).to be_falsey
+    it "now returns true for Ibycus too" do
+      expect(Language.new('grc-x-ibycus').isgreek?).to be_truthy
     end
   end
 
@@ -724,8 +724,8 @@ describe Language do
         expect(Language.new('hr').list_loader).to eq 'file=loadhyph-hr.tex'
       end
 
-      it "includes an empty line for Arabic and Farsi" do
-        expect(Language.new('ar').list_loader).to eq "file=zerohyph.tex \\\n\tfile_patterns="
+      it "no longer includes an empty line for Arabic and Farsi" do
+        expect(Language.new('ar').list_loader).to eq "file=hyph-ar.tex" # was "file=hyph-ar.tex \\\n\tfile_patterns="
       end
 
       it "includes a Lua special line for Ibycus" do
@@ -735,12 +735,19 @@ describe Language do
 
     describe '#list_run_files' do
       it "returns the list of TeX file" do
-        # puts Language.new('ka').list_run_files
+        # pending "Something changed" # ... and changed back
         expect(Language.new('ka').list_run_files).to eq ['tex/generic/hyph-utf8/loadhyph/loadhyph-ka.tex',
           'tex/generic/hyph-utf8/patterns/tex/hyph-ka.tex',
           'tex/generic/hyph-utf8/patterns/ptex/hyph-ka.t8m.tex',
-          'tex/generic/hyph-utf8/patterns/txt/hyph-ka.pat.txt',
-          'tex/generic/hyph-utf8/patterns/txt/hyph-ka.hyp.txt']
+          'tex/generic/hyph-utf8/patterns/txt/hyph-ka.pat.txt']
+      end
+
+      it "works correctly" do
+        pending "change"
+        expect(Language.new('zh-latn-pinyin').list_run_files).to eq [
+          'tex/generic/hyph-utf8/loadhyph/loadhyph-zh-latn-pinyin.tex',
+          'tex/generic/hyph-utf8/patterns/tex/hyph-zh-latn-pinyin.tex',
+          'tex/generic/hyph-utf8/patterns/txt/hyph-zh-latn-pinyin.pat.txt']
       end
     end
 
@@ -835,6 +842,16 @@ describe Language do
       end
     end
 
+    describe '#luaspecial' do
+      it "returns the Lua special comment" do
+        expect(Language.new('mn-cyrl-x-lmc').luaspecial).to be == 'disabled:only for 8bit montex with lmc encoding'
+      end
+
+      it "returns nil otherwise" do
+        expect(Language.new('mn-cyrl').luaspecial).to be_nil
+      end
+    end
+
     describe '#description' do
       it "returns the long description" do
         text = <<-EOD
@@ -963,6 +980,7 @@ describe Package do
     it "lists the run-time files" do
       # pending "it crashes ;-)"
       norwegian_run = norwegian.list_run_files
+      # pending "it crashes again ;-)" # Fixed now
       expect(norwegian_run.count).to eq 11
       expect(norwegian_run.select { |f| f =~ /tex\/hyph-[^\.]*\.tex$/ }).to eq ['tex/generic/hyph-utf8/patterns/tex/hyph-no.tex',
         'tex/generic/hyph-utf8/patterns/tex/hyph-nb.tex',
@@ -989,7 +1007,10 @@ describe Converter do
     it "reads the conversion data" do
       converter = Converter.new
       converter.read(File.join(File.expand_path(__dir__), '..', '..', '..', 'data', 'encodings', 'macedonian.dat'))
-      puts converter.instance_variable_get(:@mapping)
+      mapping = converter.instance_variable_get(:@mapping)
+      expect(mapping).to be_a Hash
+      expect(mapping.count).to be_equal 42
+      expect(mapping[248]).to be == 'ш'
     end
   end
 
@@ -997,7 +1018,9 @@ describe Converter do
     it "runs one pass through the file" do
       converter = Converter.new
       converter.read(File.join(File.expand_path(__dir__), '..', '..', '..', 'data', 'encodings', 'macedonian.dat'))
-      converter.convert(File.join(File.expand_path(__dir__), '..', '..', '..', '..', '..', '..', '..', 'old', 'other', 'mk', 'mkhyphen.tex'))
+      mkconv = converter.convert(File.join(File.expand_path(__dir__), '..', '..', '..', '..', '..', '..', '..', 'old', 'other', 'mk', 'mkhyphen.tex'))
+      expect(mkconv.each_line.count).to eq 725
+      expect(mkconv).to be =~ /1шу$/
     end
   end
 end
